function varargout = RFProcessing_GUI(varargin)
% RFPROCESSING_GUI M-menu_file for RFProcessing_GUI.fig
%      RFPROCESSING_GUI, by itself, creates a new RFPROCESSING_GUI or raises the existing
%      singleton*.
%
%      H = RFPROCESSING_GUI returns the handle to a new RFPROCESSING_GUI or the handle to
%      the existing singleton*.
%
%      RFPROCESSING_GUI('CALLBACK',hObject,eventData,handles,...) calls the
%      local
%      function named CALLBACK in RFPROCESSING_GUI.M with the given input arguments.
%
%      RFPROCESSING_GUI('Property','Value',...) creates a new RFPROCESSING_GUI or raises the
%      existing singleton*.  Starting from the left, property value pairs
%      are
%      applied to the GUI before RFProcessing_GUI_OpeningFcn gets called.  An
%      unrecognized property name or invalid value makes property application
%      stop.  All inputs are passed to RFProcessing_GUI_OpeningFcn via varargin.
%
%      *See GUI Options on GUIDE's Tools menu.  Choose "GUI allows only one
%      instance to run (singleton)".
%
% See also: GUIDE, GUIDATA, GUIHANDLES

% Copyright (c) 2010 
% Andre Dau, Athanasios Karamalis, Christian Wachinger 
% Computer Aided Medical Procedures (CAMP) 
% Technische Universitt Mnchen (TUM) 
% http://campar.in.tum.de/Students/RFtoB 
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are met:
%    * Redistributions of source code must retain the above copyright
%       notice, this list of conditions and the following disclaimer.
%    * Redistributions in binary form must reproduce the above copyright
%       notice, this list of conditions and the following disclaimer in the
%       documentation and/or other materials provided with the distribution.
%    * Neither the name of CAMP, TUM nor the
%       names of its contributors may be used to endorse or promote products
%       derived from this software without specific prior written permission.
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
% ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
% WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
% DISCLAIMED. IN NO EVENT SHALL Andre Dau, Athanasios Karamalis, Christian Wachinger BE LIABLE FOR ANY
% DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
% (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
% LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
% ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
% (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

% Edit the above text to modify the response to help RFProcessing_GUI

% Last Modified by GUIDE v2.5 12-Feb-2010 12:18:09

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @RFProcessing_GUI_OpeningFcn, ...
                   'gui_OutputFcn',  @RFProcessing_GUI_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT



% --- Executes just before RFProcessing_GUI is made visible.
function RFProcessing_GUI_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to RFProcessing_GUI (see VARARGIN)

addpath(pwd);

% Choose default command line output for RFProcessing_GUI
handles.output = hObject;

%window ratio
pos_fig = get(gcf,'Position');
handles.window_ratio = pos_fig(3)/pos_fig(4);

%maximum image size
pos = get(handles.axes_image1,'Position');
handles.previewImageMaxWidth = pos(3);
handles.previewImageMaxHeight = pos(4);

%speed of sound
handles.speedOfSound = 1540;


%transducer geometry in cm
handles.transducer_radius = 6.0;
handles.transducer_pitch = 0.04792;

%number of transducer elements
handles.transducer_number_of_elements = 128;

%transducer angle in radians
handles.transducer_angle = ((handles.transducer_number_of_elements-1) * handles.transducer_pitch) / handles.transducer_radius;

%Init filters
addpath('./filters');
[filterIds,guiMap,argsMap,procMap,editMap] = initFilters();
handles.filterIds = filterIds;
handles.createGuiMap = guiMap;
handles.formatArgsMap = argsMap;
handles.procFilterMap = procMap;
handles.editFilterMap = editMap;

%add the filters to the popup menu
popup_selectFilter = handles.popup_selectFilter;
set (popup_selectFilter,'String',handles.filterIds);
set (popup_selectFilter,'Value',1);

%display the first filter in the list
str = get(handles.popup_selectFilter,'String');
val = get(handles.popup_selectFilter,'Value');
filter = str{val};
createGuiFct = handles.createGuiMap(filter);
handles.filter_gui = createGuiFct(handles.panel_filter_options);
handles.edit_listbox = handles.listbox_filterlist1a;

% Update handles structure
guidata(hObject, handles);

% UIWAIT makes RFProcessing_GUI wait for user response (see UIRESUME)
% uiwait(handles.figure1);


% --- Outputs from this function are returned to the command line.
function varargout = RFProcessing_GUI_OutputFcn(hObject, eventdata, handles) 
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;




%%%%%%%%%%%%%%%%%%%%%
%Filter options GUI %
%%%%%%%%%%%%%%%%%%%%%

% --- Executes on button press in pb_addFilter.
function pb_addFilter_Callback(hObject, eventdata, handles)
% hObject    handle to pb_addFilter (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
str = get(handles.popup_selectFilter,'String');
val = get(handles.popup_selectFilter,'Value');

%create argument string
filter = str{val};
formatArgsFct = handles.formatArgsMap(filter);
argString = formatArgsFct(handles.filter_gui);

%determine selected listbox to which the filter is added
%default listbox
listbox = handles.listbox_filterlist1a;
step = 1;
switch(lower(get(get(handles.panel_processingSteps,'SelectedObject'),'Tag')))
    case 'radio_filterlist1a' 
        listbox = handles.listbox_filterlist1a;
        step = 1;
    case 'radio_filterlist1b' 
        listbox = handles.listbox_filterlist1b;
        step = 1;
    case 'radio_filterlist1c' 
        listbox = handles.listbox_filterlist1c;
        step = 1;
    case 'radio_filterlist2' 
        listbox = handles.listbox_filterlist2;
        step = 4;
    case 'radio_filterlist3' 
        listbox = handles.listbox_filterlist3;
        step = 5;
    case 'radio_filterlist4' 
        listbox = handles.listbox_filterlist4;
        step = 6;
    case 'radio_filterlist5' 
        listbox = handles.listbox_filterlist5;
        step = 8;
end
%add filter to listbox
str = get(listbox,'String');
str = [str;{strcat(filter,'(',argString,')')}];

set(listbox,'String',str,'Value',size(str,1));

%update states
setStates(step,'UPDATE',[1 0 0],handles);

% --- Executes on selection change in popup_selectFilter.
function popup_selectFilter_Callback(hObject, eventdata, handles)
% hObject    handle to popup_selectFilter (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = get(hObject,'String') returns popup_selectFilter contents as cell array
%        contents{get(hObject,'Value')} returns selected item from popup_selectFilter

%delete old gui
delete(get(handles.panel_filter_options,'Children'));

%create new gui
str = get(handles.popup_selectFilter,'String');
val = get(handles.popup_selectFilter,'Value');
filter = str{val};
createGuiFct = handles.createGuiMap(filter);
handles.filter_gui = createGuiFct(handles.panel_filter_options);
guidata(gcf,handles);



% --- Executes during object creation, after setting all properties.
function popup_selectFilter_CreateFcn(hObject, eventdata, handles)
% hObject    handle to popup_selectFilter (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



% --- Executes on button press in pb_edit_save.
function pb_edit_save_Callback(hObject, eventdata, handles)
% hObject    handle to pb_edit_save (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
str = get(handles.edit_listbox,'String');
val = get(handles.edit_listbox,'Value');
%create argument string
filter = str{val};
[filter,rest] = strtok(filter,'(');
formatArgsFct = handles.formatArgsMap(filter);
argString = formatArgsFct(handles.filter_gui);

%insert edited filter at the right position
result = {};
if(val>1)
    result = [result;str(1:val-1)];
end
result = [result;{strcat(filter,'(',argString,')')}];

if(val<size(str,1))
    result = [result;str(val+1:size(str))];
end        
set(handles.edit_listbox,'String',result);

%change to normal mode
handles = switchMode(handles,'on');

%update states
switch get(handles.edit_listbox,'Tag')
    case 'listbox_filterlist1a'
        setStates(1,'UPDATE',[1 0 0],handles);
    case 'listbox_filterlist1b'
        setStates(1,'UPDATE',[1 0 0],handles);
    case 'listbox_filterlist1c'
        setStates(1,'UPDATE',[1 0 0],handles);
    case 'listbox_filterlist2'
        setStates(4,'UPDATE',[1 0 0],handles);
    case 'listbox_filterlist3'
        setStates(5,'UPDATE',[1 0 0],handles);
    case 'listbox_filterlist4'
        setStates(6,'UPDATE',[1 0 0],handles);
    case 'listbox_filterlist5'
        setStates(8,'UPDATE',[1 0 0],handles);
end
guidata(gcf,handles);

% --- Executes on button press in pb_edit_cancel.
function pb_edit_cancel_Callback(hObject, eventdata, handles)
% hObject    handle to pb_edit_cancel (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
handles = switchMode(handles,'on');
guidata(gcf,handles);

%%%%%%%%%%%%%%%%%%%%
% Filter steps GUI %
%%%%%%%%%%%%%%%%%%%%

% --- Executes when selected object is changed in panel_merge.
function panel_merge_SelectionChangeFcn(hObject, eventdata, handles)
% hObject    handle to the selected object in panel_merge 
% eventdata  structure with the following fields (see UIBUTTONGROUP)
%	EventName: string 'SelectionChanged' (read only)
%	OldValue: handle of the previously selected object or empty if none was
%	selected
%	NewValue: handle of the currently selected object
% handles    structure with handles and user data (see GUIDATA)
setStates(4,'UPDATE',[1 0 0],handles);

function edit_scWidth_Callback(hObject, eventdata, handles)
% hObject    handle to edit_scWidth (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit_scWidth as text
%        str2double(get(hObject,'String')) returns contents of edit_scWidth as a double
handles.rfScWidth = str2double(get(hObject,'String'));

%update height to keep ratio
handles.rfScHeight = handles.rfScWidth/handles.rfRatio;
set(handles.edit_scHeight,'String',num2str(handles.rfScHeight));
setStates(7,'UPDATE',[1 0 0],handles);

% --- Executes during object creation, after setting all properties.
function edit_scWidth_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit_scWidth (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


function edit_scHeight_Callback(hObject, eventdata, handles)
% hObject    handle to edit_scHeight (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit_scHeight as text
%        str2double(get(hObject,'String')) returns contents of edit_scHeight as a double
handles.rfScHeight = str2double(get(hObject,'String'));

%update width to keep ratio
handles.rfScWidth = handles.rfScHeight*handles.rfRatio;
set(handles.edit_scWidth,'String',num2str(handles.rfScWidth));
setStates(7,'UPDATE',[1 0 0],handles);

% --- Executes during object creation, after setting all properties.
function edit_scHeight_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit_scHeight (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


%%%%%%%%%%%%%%%%
%DELETE-BUTTONS%
%%%%%%%%%%%%%%%%

% --- Executes on button press in button_filterlist1a_delete.
function button_filterlist1a_delete_Callback(hObject, eventdata, handles)
% hObject    handle to button_filterlist1a_delete (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
selected = get(handles.listbox_filterlist1a,'Value');
prev_str = get(handles.listbox_filterlist1a,'String');
if ~isempty(prev_str)
    prev_str(selected) = [];
    set(handles.listbox_filterlist1a, 'String', prev_str, ...
        'Value', max(1,min(selected,length(prev_str))));
    setStates(1,'UPDATE',[1 0 0],handles);
end 

% --- Executes on button press in button_filterlist1b_delete.
function button_filterlist1b_delete_Callback(hObject, eventdata, handles)
% hObject    handle to button_filterlist1b_delete (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
selected = get(handles.listbox_filterlist1b,'Value');
prev_str = get(handles.listbox_filterlist1b,'String');
if ~isempty(prev_str)
    prev_str(selected) = [];
    set(handles.listbox_filterlist1b, 'String', prev_str, ...
        'Value', max(1,min(selected,length(prev_str))));
    setStates(1,'UPDATE',[1 0 0],handles);
end 

% --- Executes on button press in button_filterlist1c_delete.
function button_filterlist1c_delete_Callback(hObject, eventdata, handles)
% hObject    handle to button_filterlist1c_delete (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
selected = get(handles.listbox_filterlist1c,'Value');
prev_str = get(handles.listbox_filterlist1c,'String');
if ~isempty(prev_str)
    prev_str(selected) = [];
    set(handles.listbox_filterlist1c, 'String', prev_str, ...
        'Value', max(1,min(selected,length(prev_str))));
    setStates(1,'UPDATE',[1 0 0],handles);
end 

% --- Executes on button press in button_filterlist2_delete.
function button_filterlist2_delete_Callback(hObject, eventdata, handles)
% hObject    handle to button_filterlist2_delete (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
selected = get(handles.listbox_filterlist2,'Value');
prev_str = get(handles.listbox_filterlist2,'String');
if ~isempty(prev_str)
    prev_str(selected) = [];
    set(handles.listbox_filterlist2, 'String', prev_str, ...
        'Value', max(1,min(selected,length(prev_str))));
    setStates(4,'UPDATE',[1 0 0],handles);
end 

% --- Executes on button press in button_filterlist3_delete.
function button_filterlist3_delete_Callback(hObject, eventdata, handles)
% hObject    handle to button_filterlist3_delete (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
selected = get(handles.listbox_filterlist3,'Value');
prev_str = get(handles.listbox_filterlist3,'String');
if ~isempty(prev_str)
    prev_str(selected) = [];
    set(handles.listbox_filterlist3, 'String', prev_str, ...
        'Value', max(1,min(selected,length(prev_str))));
    setStates(5,'UPDATE',[1 0 0],handles);
end 


% --- Executes on button press in button_filterlist4_delete.
function button_filterlist4_delete_Callback(hObject, eventdata, handles)
% hObject    handle to button_filterlist4_delete (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
selected = get(handles.listbox_filterlist4,'Value');
prev_str = get(handles.listbox_filterlist4,'String');
if ~isempty(prev_str)
    prev_str(selected) = [];
    set(handles.listbox_filterlist4, 'String', prev_str, ...
        'Value', max(1,min(selected,length(prev_str))));
    setStates(6,'UPDATE',[1 0 0],handles);
end 

% --- Executes on button press in button_filterlist5_delete.
function button_filterlist5_delete_Callback(hObject, eventdata, handles)
% hObject    handle to button_filterlist5_delete (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
selected = get(handles.listbox_filterlist5,'Value');
prev_str = get(handles.listbox_filterlist5,'String');
if ~isempty(prev_str)
    prev_str(selected) = [];
    set(handles.listbox_filterlist5, 'String', prev_str, ...
        'Value', max(1,min(selected,length(prev_str))));
    setStates(8,'UPDATE',[1 0 0],handles);
end 

%%%%%%%%%%%%%%
% UP-BUTTONS %
%%%%%%%%%%%%%%

% --- Executes on button press in pb_filterlist1a_up.
function pb_filterlist1a_up_Callback(hObject, eventdata, handles)
% hObject    handle to pb_filterlist1a_up (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
val = get(handles.listbox_filterlist1a,'Value');
str = get(handles.listbox_filterlist1a,'String');
result={};
val = val(1);
if( (val>1) && (val<=size(str,1)))
   selected = str{val};
   if(val>2)
       result = [str(1:val-2);str(val);str(val-1)];
   else
       result = [str(2);str(1)];
   end
   if(val<size(str,1))
       result=[result;str(val+1:size(str))]
   end
   set(handles.listbox_filterlist1a,'String',result,'Value',val-1);
end

% --- Executes on button press in pb_filterlist1b_up.
function pb_filterlist1b_up_Callback(hObject, eventdata, handles)
% hObject    handle to pb_filterlist1b_up (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
val = get(handles.listbox_filterlist1b,'Value');
str = get(handles.listbox_filterlist1b,'String');
result={};
val = val(1);
if( (val>1) && (val<=size(str,1)))
   selected = str{val};
   if(val>2)
       result = [str(1:val-2);str(val);str(val-1)];
   else
       result = [str(2);str(1)];
   end
   if(val<size(str,1))
       result=[result;str(val+1:size(str))]
   end
   set(handles.listbox_filterlist1b,'String',result,'Value',val-1);
end

% --- Executes on button press in pb_filterlist1c_up.
function pb_filterlist1c_up_Callback(hObject, eventdata, handles)
% hObject    handle to pb_filterlist1c_up (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
val = get(handles.listbox_filterlist1c,'Value');
str = get(handles.listbox_filterlist1c,'String');
result={};
val = val(1);
if( (val>1) && (val<=size(str,1)))
   selected = str{val};
   if(val>2)
       result = [str(1:val-2);str(val);str(val-1)];
   else
       result = [str(2);str(1)];
   end
   if(val<size(str,1))
       result=[result;str(val+1:size(str))]
   end
   set(handles.listbox_filterlist1c,'String',result,'Value',val-1);
end

% --- Executes on button press in pb_filterlist2_up.
function pb_filterlist2_up_Callback(hObject, eventdata, handles)
% hObject    handle to pb_filterlist2_up (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
val = get(handles.listbox_filterlist2,'Value');
str = get(handles.listbox_filterlist2,'String');
result={};
val = val(1);
if( (val>1) && (val<=size(str,1)))
   selected = str{val};
   if(val>2)
       result = [str(1:val-2);str(val);str(val-1)];
   else
       result = [str(2);str(1)];
   end
   if(val<size(str,1))
       result=[result;str(val+1:size(str))]
   end
   set(handles.listbox_filterlist2,'String',result,'Value',val-1);
end

% --- Executes on button press in pb_filterlist3_up.
function pb_filterlist3_up_Callback(hObject, eventdata, handles)
% hObject    handle to pb_filterlist3_up (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
val = get(handles.listbox_filterlist3,'Value');
str = get(handles.listbox_filterlist3,'String');
result={};
val = val(1);
if( (val>1) && (val<=size(str,1)))
   selected = str{val};
   if(val>2)
       result = [str(1:val-2);str(val);str(val-1)];
   else
       result = [str(2);str(1)];
   end
   if(val<size(str,1))
       result=[result;str(val+1:size(str))]
   end
   set(handles.listbox_filterlist3,'String',result,'Value',val-1);
end

% --- Executes on button press in pb_filterlist4_up.
function pb_filterlist4_up_Callback(hObject, eventdata, handles)
% hObject    handle to pb_filterlist4_up (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
val = get(handles.listbox_filterlist4,'Value');
str = get(handles.listbox_filterlist4,'String');
result={};
val = val(1);
if( (val>1) && (val<=size(str,1)))
   selected = str{val};
   if(val>2)
       result = [str(1:val-2);str(val);str(val-1)];
   else
       result = [str(2);str(1)];
   end
   if(val<size(str,1))
       result=[result;str(val+1:size(str))]
   end
   set(handles.listbox_filterlist4,'String',result,'Value',val-1);
end

% --- Executes on button press in pb_filterlist5_up.
function pb_filterlist5_up_Callback(hObject, eventdata, handles)
% hObject    handle to pb_filterlist5_up (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
val = get(handles.listbox_filterlist5,'Value');
str = get(handles.listbox_filterlist5,'String');
result={};
val = val(1);
if( (val>1) && (val<=size(str,1)))
   selected = str{val};
   if(val>2)
       result = [str(1:val-2);str(val);str(val-1)];
   else
       result = [str(2);str(1)];
   end
   if(val<size(str,1))
       result=[result;str(val+1:size(str))]
   end
   set(handles.listbox_filterlist5,'String',result,'Value',val-1);
end

%%%%%%%%%%%%%%
% DOWN-BUTTONS %
%%%%%%%%%%%%%%

% --- Executes on button press in pb_filterlist1a_down.
function pb_filterlist1a_down_Callback(hObject, eventdata, handles)
% hObject    handle to pb_filterlist1a_down (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
val = get(handles.listbox_filterlist1a,'Value');
str = get(handles.listbox_filterlist1a,'String');
result={};
val = val(1);
if( (val>0) && (val<size(str,1)))
   selected = str{val};
   if(val>1)
       result = [str(1:val-1);str(val+1);str(val)];
   else
       result = [str(2);str(1)];
   end
   if(val<size(str,1)-1)
       result=[result;str(val+2:size(str))]
   end
   set(handles.listbox_filterlist1a,'String',result,'Value',val+1);
end


% --- Executes on button press in pb_filterlist1b_down.
function pb_filterlist1b_down_Callback(hObject, eventdata, handles)
% hObject    handle to pb_filterlist1b_down (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
val = get(handles.listbox_filterlist1b,'Value');
str = get(handles.listbox_filterlist1b,'String');
result={};
val = val(1);
if( (val>0) && (val<size(str,1)))
   selected = str{val};
   if(val>1)
       result = [str(1:val-1);str(val+1);str(val)];
   else
       result = [str(2);str(1)];
   end
   if(val<size(str,1)-1)
       result=[result;str(val+2:size(str))]
   end
   set(handles.listbox_filterlist1b,'String',result,'Value',val+1);
end

% --- Executes on button press in pb_filterlist1c_down.
function pb_filterlist1c_down_Callback(hObject, eventdata, handles)
% hObject    handle to pb_filterlist1c_down (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
val = get(handles.listbox_filterlist1c,'Value');
str = get(handles.listbox_filterlist1c,'String');
result={};
val = val(1);
if( (val>0) && (val<size(str,1)))
   selected = str{val};
   if(val>1)
       result = [str(1:val-1);str(val+1);str(val)];
   else
       result = [str(2);str(1)];
   end
   if(val<size(str,1)-1)
       result=[result;str(val+2:size(str))]
   end
   set(handles.listbox_filterlist1c,'String',result,'Value',val+1);
end

% --- Executes on button press in pb_filterlist2_down.
function pb_filterlist2_down_Callback(hObject, eventdata, handles)
% hObject    handle to pb_filterlist2_down (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
val = get(handles.listbox_filterlist2,'Value');
str = get(handles.listbox_filterlist2,'String');
result={};
val = val(1);
if( (val>0) && (val<size(str,1)))
   selected = str{val};
   if(val>1)
       result = [str(1:val-1);str(val+1);str(val)];
   else
       result = [str(2);str(1)];
   end
   if(val<size(str,1)-1)
       result=[result;str(val+2:size(str))]
   end
   set(handles.listbox_filterlist2,'String',result,'Value',val+1);
end

% --- Executes on button press in pb_filterlist3_down.
function pb_filterlist3_down_Callback(hObject, eventdata, handles)
% hObject    handle to pb_filterlist3_down (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
val = get(handles.listbox_filterlist3,'Value');
str = get(handles.listbox_filterlist3,'String');
result={};
val = val(1);
if( (val>0) && (val<size(str,1)))
   selected = str{val};
   if(val>1)
       result = [str(1:val-1);str(val+1);str(val)];
   else
       result = [str(2);str(1)];
   end
   if(val<size(str,1)-1)
       result=[result;str(val+2:size(str))]
   end
   set(handles.listbox_filterlist3,'String',result,'Value',val+1);
end

% --- Executes on button press in pb_filterlist4_down.
function pb_filterlist4_down_Callback(hObject, eventdata, handles)
% hObject    handle to pb_filterlist4_down (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
val = get(handles.listbox_filterlist4,'Value');
str = get(handles.listbox_filterlist4,'String');
result={};
val = val(1);
if( (val>0) && (val<size(str,1)))
   selected = str{val};
   if(val>1)
       result = [str(1:val-1);str(val+1);str(val)];
   else
       result = [str(2);str(1)];
   end
   if(val<size(str,1)-1)
       result=[result;str(val+2:size(str))]
   end
   set(handles.listbox_filterlist4,'String',result,'Value',val+1);
end

% --- Executes on button press in pb_filterlist5_down.
function pb_filterlist5_down_Callback(hObject, eventdata, handles)
% hObject    handle to pb_filterlist5_down (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
val = get(handles.listbox_filterlist5,'Value');
str = get(handles.listbox_filterlist5,'String');
result={};
val = val(1);
if( (val>0) && (val<size(str,1)))
   selected = str{val};
   if(val>1)
       result = [str(1:val-1);str(val+1);str(val)];
   else
       result = [str(2);str(1)];
   end
   if(val<size(str,1)-1)
       result=[result;str(val+2:size(str))]
   end
   set(handles.listbox_filterlist5,'String',result,'Value',val+1);
end


%%%%%%%%%%%%%%%%
% EDIT-BUTTONS %
%%%%%%%%%%%%%%%%
% --- Executes on button press in pb_filterlist1a_edit.
function pb_filterlist1a_edit_Callback(hObject, eventdata, handles)
% hObject    handle to pb_filterlist1a_edit (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
handles.edit_listbox = handles.listbox_filterlist1a;
handles = switchMode(handles,'off');
guidata(gcf,handles);

% --- Executes on button press in pb_filterlist1b_edit.
function pb_filterlist1b_edit_Callback(hObject, eventdata, handles)
% hObject    handle to pb_filterlist1b_edit (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
handles.edit_listbox = handles.listbox_filterlist1b;
handles = switchMode(handles,'off');
guidata(gcf,handles);

% --- Executes on button press in pb_filterlist1c_edit.
function pb_filterlist1c_edit_Callback(hObject, eventdata, handles)
% hObject    handle to pb_filterlist1c_edit (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
handles.edit_listbox = handles.listbox_filterlist1c;
handles = switchMode(handles,'off');
guidata(gcf,handles);

% --- Executes on button press in pb_filterlist2_edit.
function pb_filterlist2_edit_Callback(hObject, eventdata, handles)
% hObject    handle to pb_filterlist2_edit (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
handles.edit_listbox = handles.listbox_filterlist2;
handles = switchMode(handles,'off');
guidata(gcf,handles);

% --- Executes on button press in pb_filterlist3_edit.
function pb_filterlist3_edit_Callback(hObject, eventdata, handles)
% hObject    handle to pb_filterlist3_edit (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
handles.edit_listbox = handles.listbox_filterlist3;
handles = switchMode(handles,'off');
guidata(gcf,handles);

% --- Executes on button press in pb_filterlist4_edit.
function pb_filterlist4_edit_Callback(hObject, eventdata, handles)
% hObject    handle to pb_filterlist4_edit (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
handles.edit_listbox = handles.listbox_filterlist4;
handles = switchMode(handles,'off');
guidata(gcf,handles);

% --- Executes on button press in pb_filterlist5_edit.
function pb_filterlist5_edit_Callback(hObject, eventdata, handles)
% hObject    handle to pb_filterlist5_edit (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
handles.edit_listbox = handles.listbox_filterlist5;
handles = switchMode(handles,'off');
guidata(gcf,handles);


%%%%%%%%%%%%%%%%%%%%%
% ENABLE-CHECKBOXES %
%%%%%%%%%%%%%%%%%%%%%

% --- Executes on button press in check_performStep1a.
function check_performStep1a_Callback(hObject, eventdata, handles)
% hObject    handle to check_performStep1a (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of check_performStep1a
setStates(1,'UPDATE',[1 0 0],handles);

% --- Executes on button press in check_performStep1b.
function check_performStep1b_Callback(hObject, eventdata, handles)
% hObject    handle to check_performStep1b (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of check_performStep1b
setStates(1,'UPDATE',[1 0 0],handles);


% --- Executes on button press in check_performStep1c.
function check_performStep1c_Callback(hObject, eventdata, handles)
% hObject    handle to check_performStep1c (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of check_performStep1c
setStates(1,'UPDATE',[1 0 0],handles);

% --- Executes on button press in check_performStep2.
function check_performStep2_Callback(hObject, eventdata, handles)
% hObject    handle to check_performStep2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of check_performStep2
setStates(4,'UPDATE',[1 0 0],handles);

% --- Executes on button press in check_performStep3.
function check_performStep3_Callback(hObject, eventdata, handles)
% hObject    handle to check_performStep3 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of check_performStep3
setStates(5,'UPDATE',[1 0 0],handles);

% --- Executes on button press in check_performStep4.
function check_performStep4_Callback(hObject, eventdata, handles)
% hObject    handle to check_performStep4 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of check_performStep4
setStates(6,'UPDATE',[1 0 0],handles);

% --- Executes on button press in check_performStep5.
function check_performStep5_Callback(hObject, eventdata, handles)
% hObject    handle to check_performStep5 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of check_performStep5
setStates(7,'UPDATE',[1 0 0],handles);

% --- Executes on button press in check_performStep6.
function check_performStep6_Callback(hObject, eventdata, handles)
% hObject    handle to check_performStep6 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of check_performStep6
setStates(8,'UPDATE',[1 0 0],handles);



%%%%%%%%%%%%%%%%%
% Perform Steps %
%%%%%%%%%%%%%%%%%
function newHandles = performSteps(handles)

%start with the raw rf image
processedImage = handles.images{7};

%allows indexed access to gui elements
listboxes = [handles.listbox_filterlist1a handles.listbox_filterlist1b handles.listbox_filterlist1c handles.listbox_filterlist2,handles.listbox_filterlist3,handles.listbox_filterlist4,handles.listbox_filterlist5];
states = [handles.text_state1a handles.text_state1b handles.text_state1c handles.text_state2 handles.text_state3 handles.text_state4 handles.text_state5 handles.text_state6];
checkboxes = [handles.check_performStep1a handles.check_performStep1b handles.check_performStep1c handles.check_performStep2 handles.check_performStep3 handles.check_performStep4 handles.check_performStep5 handles.check_performStep6];

%six steps
for(stepIndex = 1:6)
    %all other steps than rf filtering
    if(stepIndex>1)
        %if step should be performed
        if get(checkboxes(stepIndex+2),'Value')==1
            %if somethin has changed since the last update
            if strcmp(get(states(stepIndex+2),'String'),'UPDATE')
                %select correct listbox
                listbox = listboxes(7);
                if(stepIndex < 5)
                    listbox = listboxes(stepIndex+2);
                end
                %other than scan conversion
                if(stepIndex~=5)
                    filterList = get(listbox,'String');
                    %special treatment for envelope detection if signals
                    %are merge after the envelope detection
                    if (stepIndex~=2)||(get(handles.radio_merge_before,'Value')==1)
                        for(i = 1:size(filterList))
                            filterString = filterList{i};
                            [filter,argString]  = strtok(filterString,'(');
                            argString = argString(2:end-1);
                            procFct = handles.procFilterMap(filter);
                            processedImage = procFct(processedImage,argString,handles);
                        end
                    %envelope detection before merging signals
                    else
                        procImg=[handles.images(8) handles.images(9) handles.images(10)];
                        processedImage = 0;
                        %use raw rf image if no rf filters were applied
                        if isempty(procImg{1})
                            if isempty(procImg{2})
                                if isempty(procImg{3})
                                    procImg{1} = handles.images{7};
                                end
                            end
                        end
                        count = 0;
                        for j=1:3
                            if ~isempty(procImg{j})
                                count = count+1;
                                for(i = 1:size(filterList))
                                    filterString = filterList{i};
                                    [filter,argString]  = strtok(filterString,'(');
                                    argString = argString(2:end-1);
                                    procFct = handles.procFilterMap(filter);
                                    procImg{j} = procFct(procImg{j},argString,handles);
                                end
                                processedImage = processedImage + procImg{j};
                            end
                        end
                        processedImage = processedImage/count;
                    end
                %scan conversion
                else
                    probe.id = handles.rfHdr.probe;
                    probe.type = 0;
                    probe.elements = handles.transducer_number_of_elements;
                    probe.radius = handles.transducer_angle/pi*180.0*1000;
                    probe.pitch = handles.transducer_pitch*10000;
                    probe.maxsteerangle = 0;
                    probe.transmitoffset = 0;
                    
                    [processedImage,res] =  scan_conversion(processedImage,handles.rfSamplingFreq,probe,handles.rfScWidth,handles.rfScHeight,handles.rfLineDensity);
                    
                    %flip image
                    processedImage = fliplr(processedImage);
                    
                    %update image geometry
                    handles.rfWidth = size(processedImage,2)*res/10000;
                    handles.rfHeight = size(processedImage,1)*res/10000;
                    handles.rfRatio = handles.rfWidth/handles.rfHeight;
                end
                
                %update states
                set(states(stepIndex+2),'String','DONE','ForegroundColor',[0 1 0]);
            %if nothing has changed use stored image
            else
                processedImage = handles.images{stepIndex};
            end
        %if step is not enabled then use image of previous step
        else
            if(stepIndex == 1)
                prev = 7;
            else
                prev = stepIndex-1;
            end
            processedImage = handles.images{prev};
        end
    %rf filtering
    else
        processedImage = 0;
        count = 0;
        for j=1:3
           if (get(checkboxes(j),'Value')==1) && strcmp(get(states(j),'String'),'UPDATE')
                listbox = listboxes(j);
                handles.images{7+j} = handles.images{7};
                filterList = get(listbox,'String');
                for(i = 1:size(filterList))
                    filterString = filterList{i};
                    [filter,argString]  = strtok(filterString,'(');
                    argString = argString(2:end-1);
                    procFct = handles.procFilterMap(filter);
                    handles.images{7+j} = procFct(handles.images{7+j},argString,handles);
                end
                set(states(j),'String','DONE','ForegroundColor',[0 1 0]);
                processedImage = processedImage + handles.images{7+j};
                count = count+1;
           elseif (get(checkboxes(j),'Value')==0)
                handles.images{7+j} = [];
           else
               processedImage = processedImage + handles.images{7+j};
               count = count+1;
           end
        end
        %if no filter was selected then use raw rf image
        if (count == 0)
            processedImage = handles.images{7};
        %normalize
        else
            processedImage = processedImage/count;
        end
    end
    handles.images(stepIndex) = {processedImage};
end
%update preview images
panel_selStep1_SelectionChangeFcn(get(handles.panel_selStep1,'SelectedObject'),[],handles);
panel_selStep2_SelectionChangeFcn(get(handles.panel_selStep2,'SelectedObject'),[],handles);
newHandles = handles;

% --- Executes on button press in button_performSteps.
function button_performSteps_Callback(hObject, eventdata, handles)
% hObject    handle to button_performSteps (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
guidata(gcf,performSteps(handles));

% returns the selected preview image
function [image] = getSelectedImage(panel,handles)
switch(get(get(panel,'SelectedObject'),'Tag'))
    case {'radio_sel1step0','radio_sel2step0'}
        image=handles.images{7};
    case {'radio_sel1step1','radio_sel2step1'}
        image=handles.images{1};
    case {'radio_sel1step2','radio_sel2step2'}
        image=handles.images{2};
    case {'radio_sel1step3','radio_sel2step3'}
        image=handles.images{3};
    case {'radio_sel1step4','radio_sel2step4'}
        image=handles.images{4};
    case {'radio_sel1step5','radio_sel2step5'}
        image=handles.images{5};
    case {'radio_sel1step6','radio_sel2step6'}
        image=handles.images{6};
end

% --- Executes on button press in button_allSteps.
function button_allSteps_Callback(hObject, eventdata, handles)
% hObject    handle to button_allSteps (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
handles = performSteps(handles);
guidata(gcf,handles);
RFProcessing_result(handles.images,handles.rfWidth,handles.rfHeight,handles.rfRatio);


%%%%%%%%%%%%%%%%%
% Open images   %
%%%%%%%%%%%%%%%%%

% --- Executes on button press in pb_openRf.
function pb_openRf_Callback(hObject, eventdata, handles)
% hObject    handle to pb_openRf (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

[file,path] = uigetfile({'*.rf','RF images';'*.*','All files'},'Open RF image');
if(~(file==0))
[im,hdr] = RPread_noTag(fullfile(path,file));

%get rf information
handles.rfNumberOfScanlines = hdr.w;
handles.rfLineDensity = hdr.ld;
handles.rfSectorSize = hdr.w/hdr.ld;
handles.rfSectorAngle = handles.rfSectorSize * handles.transducer_angle;
handles.rfSamplingFreq = hdr.sf;
handles.rfNumberOfSamples = hdr.h; 
handles.rfDepth = handles.rfNumberOfSamples * (100 * handles.speedOfSound) / (2.0 * handles.rfSamplingFreq);
handles.rfLineDensity = hdr.ld;
handles.rfHdr = hdr;

%calculate approximate image size after scanner conversion
a = handles.rfDepth+handles.transducer_radius;
c = handles.transducer_radius*sqrt(2*(1-cos(handles.rfSectorAngle)));

%cosine rule, isoscele triangle
width = c*(a/handles.transducer_radius);
height = a-sqrt(handles.transducer_radius*handles.transducer_radius-1/4.0*c*c);

%update image geomtry
handles.rfRatio = width/height;
handles.rfWidth = width;
handles.rfHeight = height;

%init stored images of the processing steps
handles.images = [];
for i=1:10;
    handles.images{i} = im(:,:,1);
end

%default image size after scan conversion
handles.rfScWidth = 1024;
handles.rfScHeight = handles.rfScWidth/handles.rfRatio;

figPos = get(handles.master_panel,'Position');
ratio = handles.rfRatio;
width = min(handles.previewImageMaxWidth*figPos(3),handles.previewImageMaxHeight*figPos(4)*ratio)/figPos(3);
height = min(handles.previewImageMaxHeight*figPos(4),handles.previewImageMaxWidth*figPos(3)/ratio)/figPos(4);

pos = get(handles.axes_image1,'Position');
set(handles.axes_image1,'Position',[pos(1:2) width height]);
axes(handles.axes_image1);
colormap(gray(256));
image(handles.images{7},'Parent',handles.axes_image1);
axis off;

pos = get(handles.axes_image2,'Position');
set(handles.axes_image2,'Position',[pos(1:2) width height]);
axes(handles.axes_image2);
colormap(gray(256));
image(handles.images{7},'Parent',handles.axes_image2);
axis off;

set(handles.button_openRefImage,'Enable','on');
set(handles.panel_selStep1,'Visible','on');
set(handles.panel_selStep2,'Visible','on');
set(handles.button_allSteps,'Enable','on');
set(handles.button_performSteps,'Enable','on');
set(handles.button_analyseImage,'Enable','on');
set(handles.edit_scWidth,'String',num2str(handles.rfScWidth));
set(handles.edit_scHeight,'String',num2str(handles.rfScHeight));
set(handles.edit_scWidth,'Enable','on');
set(handles.edit_scHeight,'Enable','on');
set(handles.menu_setting,'Enable','on');
set(handles.pb_addFilter,'Enable','on');
guidata(gcf,handles);
end

% --- Executes on button press in button_openRefImage.
function button_openRefImage_Callback(hObject, eventdata, handles)
% hObject    handle to button_openRefImage (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
[file,path] = uigetfile({'*.*','All files'},'Open RF image');
if(~(file==0))
[im,hdr] = RPread_noTag(fullfile(path,file));
img = im(:,:,1);
handles.refImg = img;
pos = get(handles.axes_image3,'Position');

figPos = get(handles.master_panel,'Position');


handles.refImgRatio = handles.rfRatio;
%scan converted images have own ratio
if(~(strcmp(file(end-2:end),'bpr')||strcmp(file(end-1:end),'rf')))
    handles.refImgRatio = size(img,2)/size(img,1);
end
width = min(handles.previewImageMaxWidth*figPos(3),handles.previewImageMaxHeight*figPos(4)*handles.refImgRatio)/figPos(3);
height = min(handles.previewImageMaxHeight*figPos(4),handles.previewImageMaxWidth*figPos(3)/handles.refImgRatio)/figPos(4);
set(handles.axes_image3,'Position',[pos(1:2) width height]);
axes(handles.axes_image3);
colormap(gray(256));
imagesc(img,'Parent',handles.axes_image3);
axis off;
guidata(gcf,handles);
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Selection Panels for image steps%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% --- Executes when selected object is changed in panel_selStep1.
function panel_selStep1_SelectionChangeFcn(hObject, eventdata, handles)
% hObject    handle to the selected object in panel_selStep1 
% eventdata  structure with the following fields (see UIBUTTONGROUP)
%	EventName: string 'SelectionChanged' (read only)
%	OldValue: handle of the previously selected object or empty if none was selected
%	NewValue: handle of the currently selected object
% handles    structure with handles and user data (see GUIDATA)
if isfield(handles,'rfRatio')
axes(handles.axes_image1);
colormap(gray(256));
image(getSelectedImage(handles.panel_selStep1,handles),'Parent',handles.axes_image1);
axis off;
end


% --- Executes when selected object is changed in panel_selStep2.
function panel_selStep2_SelectionChangeFcn(hObject, eventdata, handles)
% hObject    handle to the selected object in panel_selStep2 
% eventdata  structure with the following fields (see UIBUTTONGROUP)
%	EventName: string 'SelectionChanged' (read only)
%	OldValue: handle of the previously selected object or empty if none was selected
%	NewValue: handle of the currently selected object
% handles    structure with handles and user data (see GUIDATA)
if isfield(handles,'rfRatio')
axes(handles.axes_image2);
colormap(gray(256));
image(getSelectedImage(handles.panel_selStep2,handles),'Parent',handles.axes_image2);
axis off;
end


%%%%%%%%%%%%%%%%%%%
% State Functions %
%%%%%%%%%%%%%%%%%%%

%set states of all following steps
function setStates(index,state,color,handles)
states = [handles.text_state1a handles.text_state1b handles.text_state1c handles.text_state2 handles.text_state3 handles.text_state4 handles.text_state5 handles.text_state6];
for i=index:8
    set(states(i),'String',state,'ForegroundColor',color);
end

%enable/disable gui elements according to 
%operating mode (edit filter or normal)
function [newHandle] = switchMode(handles,mode)

set(handles.pb_filterlist1a_up,'Enable',mode);
set(handles.pb_filterlist1b_up,'Enable',mode);
set(handles.pb_filterlist1c_up,'Enable',mode);
set(handles.pb_filterlist2_up,'Enable',mode);
set(handles.pb_filterlist3_up,'Enable',mode);
set(handles.pb_filterlist4_up,'Enable',mode);
set(handles.pb_filterlist5_up,'Enable',mode);
set(handles.pb_filterlist1a_down,'Enable',mode);
set(handles.pb_filterlist1b_down,'Enable',mode);
set(handles.pb_filterlist1c_down,'Enable',mode);
set(handles.pb_filterlist2_down,'Enable',mode);
set(handles.pb_filterlist3_down,'Enable',mode);
set(handles.pb_filterlist4_down,'Enable',mode);
set(handles.pb_filterlist5_down,'Enable',mode);
set(handles.pb_filterlist1a_edit,'Enable',mode);
set(handles.pb_filterlist1b_edit,'Enable',mode);
set(handles.pb_filterlist1c_edit,'Enable',mode);
set(handles.pb_filterlist2_edit,'Enable',mode);
set(handles.pb_filterlist3_edit,'Enable',mode);
set(handles.pb_filterlist4_edit,'Enable',mode);
set(handles.pb_filterlist5_edit,'Enable',mode);
set(handles.button_filterlist1a_delete,'Enable',mode);
set(handles.button_filterlist1b_delete,'Enable',mode);
set(handles.button_filterlist1c_delete,'Enable',mode);
set(handles.button_filterlist2_delete,'Enable',mode);
set(handles.button_filterlist3_delete,'Enable',mode);
set(handles.button_filterlist4_delete,'Enable',mode);
set(handles.button_filterlist5_delete,'Enable',mode);
set(handles.radio_filterlist1a,'Enable',mode);
set(handles.radio_filterlist1b,'Enable',mode);
set(handles.radio_filterlist1c,'Enable',mode);
set(handles.radio_filterlist2,'Enable',mode);
set(handles.radio_filterlist3,'Enable',mode);
set(handles.radio_filterlist4,'Enable',mode);
set(handles.listbox_filterlist5,'Enable',mode);
set(handles.listbox_filterlist1a,'Enable',mode);
set(handles.listbox_filterlist1b,'Enable',mode);
set(handles.listbox_filterlist1c,'Enable',mode);
set(handles.listbox_filterlist2,'Enable',mode);
set(handles.listbox_filterlist3,'Enable',mode);
set(handles.listbox_filterlist4,'Enable',mode);
set(handles.listbox_filterlist5,'Enable',mode);
set(handles.pb_openRf,'Enable',mode);
set(handles.button_allSteps,'Enable',mode);
set(handles.button_performSteps,'Enable',mode);

set(handles.pb_addFilter,'Visible',mode);
set(handles.popup_selectFilter,'Visible',mode);

switch(mode)
    case 'off'
        set(handles.pb_edit_save,'Visible','on');
        set(handles.pb_edit_cancel,'Visible','on');
        
        str = get(handles.edit_listbox,'String');
        val = get(handles.edit_listbox,'Value');
        filterString = str{val};
        
        [filter,rest] = strtok(filterString,'(');
        
        delete(get(handles.panel_filter_options,'Children'));
        createGuiFct = handles.createGuiMap(filter);
        handles.filter_gui = createGuiFct(handles.panel_filter_options);
        editFct = handles.editFilterMap(filter);
        editFct(handles.filter_gui,rest(2:end-1));
        
    case 'on'
        set(handles.pb_edit_save,'Visible','off');
        set(handles.pb_edit_cancel,'Visible','off');
        
        delete(get(handles.panel_filter_options,'Children'));
        str = get(handles.popup_selectFilter,'String');
        val = get(handles.popup_selectFilter,'Value');
        filter = str{val};
        createGuiFct = handles.createGuiMap(filter);
        handles.filter_gui = createGuiFct(handles.panel_filter_options);
       
end
newHandle = handles;

%%%%%%%%%%%%%%%%%%
% Open-Save Menu %
%%%%%%%%%%%%%%%%%%

% --------------------------------------------------------------------
function menu_setting_open_Callback(hObject, eventdata, handles)
% hObject    handle to menu_setting_open (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
[FileName,PathName] = uigetfile('.rfs','Open filter settings');
if(FileName ~= 0)
    %init states
    setStates(1,'UPDATE',[1 0 0],handles);
    file = fopen(fullfile(PathName,FileName));
    listboxes = [handles.listbox_filterlist1a,handles.listbox_filterlist1b,handles.listbox_filterlist1c,handles.listbox_filterlist2,0,handles.listbox_filterlist3,handles.listbox_filterlist4,0,handles.listbox_filterlist5];

    for i= 1:size(listboxes,2)
        fscanf(file,'(Begin%*d)\n',1);
        %scan conversion
        if i==8
            res = fscanf(file,'%f %f\n');
            handles.rfScWidth = res(1);
            handles.rfScHeight = res(2);
            set(handles.edit_scWidth,'String',num2str(res(1)));
            set(handles.edit_scHeight,'String',num2str(res(2)));
            fscanf(file,'(End%*d)\n',1);
        %merge before/after setting
        elseif i == 5
            res = fgetl(file);
            if strcmp(res,'before')
                set(handles.radio_merge_before,'Value',1);
            else
                set(handles.radio_merge_after,'Value',1);
            end
            fscanf(file,'(End%*d)\n',1);
        else
            line = fgetl(file);
            res = {};
            while(~strcmp(line,sprintf('(End%d)',i)))
                res = [res;{line}];
                line = fgetl(file);
                if(line == -1)
                    return;
                end
            end
            set(listboxes(i),'String',res,'Value',1.0);
        end
    end
    fclose(file);
    guidata(gcf,handles);
end

% --------------------------------------------------------------------
function menu_setting_save_Callback(hObject, eventdata, handles)
% hObject    handle to menu_setting_save (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
[FileName,PathName] = uiputfile('.rfs','Save filter settings');
if(FileName ~= 0)
    file = fopen(fullfile(PathName,FileName),'w+');
    listboxes = [handles.listbox_filterlist1a,handles.listbox_filterlist1b,handles.listbox_filterlist1c,handles.listbox_filterlist2,0,handles.listbox_filterlist3,handles.listbox_filterlist4,0,handles.listbox_filterlist5];
    for i= 1:size(listboxes,2)
        fprintf(file,'(Begin%d)\n',i);
        %scan conversion
        if i==8
            fprintf(file,'%d %d\n',handles.rfScWidth,handles.rfScHeight);
        %merge before/after
        elseif i == 5
            if get(handles.radio_merge_before,'Value') == 1
                fprintf(file,'before\n');
            else
                fprintf(file,'after\n');
            end
        else
            filters = get(listboxes(i),'String');
            for j = 1:size(filters,1)
                fprintf(file,'%s\n',filters{j,1});
            end
        end
        fprintf(file,'(End%d)\n',i);

    end
    fclose(file);
end


%%%%%%%%%%
% Others %
%%%%%%%%%%
% --- Executes on button press in button_analyseImage.
function button_analyseImage_Callback(hObject, eventdata, handles)
% hObject    handle to button_analyseImage (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
RFProcessing_details(handles);



% --- Executes during object creation, after setting all properties.
function panel_selStep2_CreateFcn(hObject, eventdata, handles)
% hObject    handle to panel_selStep2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% --------------------------------------------------------------------
function menu_file_Callback(hObject, eventdata, handles)
% hObject    handle to menu_file (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --------------------------------------------------------------------
function menu_file_open_Callback(hObject, eventdata, handles)
% hObject    handle to menu_file_open (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --------------------------------------------------------------------
function menu_file_save_Callback(hObject, eventdata, handles)
% hObject    handle to menu_file_save (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --- Executes on selection change in listbox_filterlist1a.
function listbox_filterlist1a_Callback(hObject, eventdata, handles)
% hObject    handle to listbox_filterlist1a (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = get(hObject,'String') returns listbox_filterlist1a contents as cell array
%        contents{get(hObject,'Value')} returns selected item from listbox_filterlist1a


% --- Executes during object creation, after setting all properties.
function listbox_filterlist1a_CreateFcn(hObject, eventdata, handles)
% hObject    handle to listbox_filterlist1a (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: listbox controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in listbox_filterlist1b.
function listbox_filterlist1b_Callback(hObject, eventdata, handles)
% hObject    handle to listbox_filterlist1b (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = get(hObject,'String') returns listbox_filterlist1b contents as cell array
%        contents{get(hObject,'Value')} returns selected item from listbox_filterlist1b


% --- Executes during object creation, after setting all properties.
function listbox_filterlist1b_CreateFcn(hObject, eventdata, handles)
% hObject    handle to listbox_filterlist1b (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: listbox controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in listbox_filterlist1c.
function listbox_filterlist1c_Callback(hObject, eventdata, handles)
% hObject    handle to listbox_filterlist1c (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = get(hObject,'String') returns listbox_filterlist1c contents as cell array
%        contents{get(hObject,'Value')} returns selected item from listbox_filterlist1c


% --- Executes during object creation, after setting all properties.
function listbox_filterlist1c_CreateFcn(hObject, eventdata, handles)
% hObject    handle to listbox_filterlist1c (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: listbox controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on button press in radiobutton1.
function radiobutton1_Callback(hObject, eventdata, handles)
% hObject    handle to radiobutton1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of radiobutton1


% --- Executes on button press in radio_filterlist1a.
function radio_filterlist1a_Callback(hObject, eventdata, handles)
% hObject    handle to radio_filterlist1a (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of radio_filterlist1a

% --- Executes on selection change in listbox_filterlist2.
function listbox_filterlist2_Callback(hObject, eventdata, handles)
% hObject    handle to listbox_filterlist2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = get(hObject,'String') returns listbox_filterlist2 contents as cell array
%        contents{get(hObject,'Value')} returns selected item from listbox_filterlist2


% --- Executes during object creation, after setting all properties.
function listbox_filterlist2_CreateFcn(hObject, eventdata, handles)
% hObject    handle to listbox_filterlist2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: listbox controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in radio_filterlist2.
function radio_filterlist2_Callback(hObject, eventdata, handles)
% hObject    handle to radio_filterlist2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of radio_filterlist2




% --- Executes on selection change in listbox_filterlist3.
function listbox_filterlist3_Callback(hObject, eventdata, handles)
% hObject    handle to listbox_filterlist3 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = get(hObject,'String') returns listbox_filterlist3 contents as cell array
%        contents{get(hObject,'Value')} returns selected item from listbox_filterlist3


% --- Executes during object creation, after setting all properties.
function listbox_filterlist3_CreateFcn(hObject, eventdata, handles)
% hObject    handle to listbox_filterlist3 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: listbox controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in radio_filterlist3.
function radio_filterlist3_Callback(hObject, eventdata, handles)
% hObject    handle to radio_filterlist3 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of radio_filterlist3




% --- Executes on selection change in listbox_filterlist4.
function listbox_filterlist4_Callback(hObject, eventdata, handles)
% hObject    handle to listbox_filterlist4 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = get(hObject,'String') returns listbox_filterlist4 contents as cell array
%        contents{get(hObject,'Value')} returns selected item from listbox_filterlist4


% --- Executes during object creation, after setting all properties.
function listbox_filterlist4_CreateFcn(hObject, eventdata, handles)
% hObject    handle to listbox_filterlist4 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: listbox controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in radio_filterlist4.
function radio_filterlist4_Callback(hObject, eventdata, handles)
% hObject    handle to radio_filterlist4 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of radio_filterlist4



% --- Executes on selection change in listbox_filterlist5.
function listbox_filterlist5_Callback(hObject, eventdata, handles)
% hObject    handle to listbox_filterlist5 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = get(hObject,'String') returns listbox_filterlist5 contents as cell array
%        contents{get(hObject,'Value')} returns selected item from listbox_filterlist5


% --- Executes during object creation, after setting all properties.
function listbox_filterlist5_CreateFcn(hObject, eventdata, handles)
% hObject    handle to listbox_filterlist5 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: listbox controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in radio_filterlist5.
function radio_filterlist5_Callback(hObject, eventdata, handles)
% hObject    handle to radio_filterlist5 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of radio_filterlist5




% --- Executes on selection change in popup_filter_options.
function popup_filter_options_Callback(hObject, eventdata, handles)
% hObject    handle to popup_filter_options (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = get(hObject,'String') returns popup_filter_options contents as cell array
%        contents{get(hObject,'Value')} returns selected item from popup_filter_options


% --- Executes during object creation, after setting all properties.
function popup_filter_options_CreateFcn(hObject, eventdata, handles)
% hObject    handle to popup_filter_options (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function edit1_Callback(hObject, eventdata, handles)
% hObject    handle to edit1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit1 as text
%        str2double(get(hObject,'String')) returns contents of edit1 as a double


% --- Executes during object creation, after setting all properties.
function edit1_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function edit2_Callback(hObject, eventdata, handles)
% hObject    handle to edit2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit2 as text
%        str2double(get(hObject,'String')) returns contents of edit2 as a double


% --- Executes during object creation, after setting all properties.
function edit2_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function edit3_Callback(hObject, eventdata, handles)
% hObject    handle to edit3 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit3 as text
%        str2double(get(hObject,'String')) returns contents of edit3 as a double


% --- Executes during object creation, after setting all properties.
function edit3_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit3 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function edit4_Callback(hObject, eventdata, handles)
% hObject    handle to edit4 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit4 as text
%        str2double(get(hObject,'String')) returns contents of edit4 as a double


% --- Executes during object creation, after setting all properties.
function edit4_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit4 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes during object creation, after setting all properties.
function figure1_CreateFcn(hObject, eventdata, handles)
% hObject    handle to figure1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called


% --- Executes when figure1 is resized.
function figure1_ResizeFcn(hObject, eventdata, handles)
% hObject    handle to figure1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
if isfield(handles,'window_ratio')
figPos = get(gcf,'Position');
set(handles.master_panel,'Position',[0,0, min(figPos(3),figPos(4)*handles.window_ratio),min(figPos(4),figPos(3)/handles.window_ratio)]);
end
if(isfield(handles,'rfRatio'))
figPos = get(handles.master_panel,'Position');
ratio = handles.rfRatio;
width = min(handles.previewImageMaxWidth*figPos(3),handles.previewImageMaxHeight*figPos(4)*ratio)/figPos(3);
height = min(handles.previewImageMaxHeight*figPos(4),handles.previewImageMaxWidth*figPos(3)/ratio)/figPos(4);

pos = get(handles.axes_image1,'Position');
set(handles.axes_image1,'Position',[pos(1:2) width height]);

pos = get(handles.axes_image2,'Position');
set(handles.axes_image2,'Position',[pos(1:2) width height]);

if(isfield(handles,'refImgRatio'))
    ratio = handles.refImgRatio;
    width = min(handles.previewImageMaxWidth*figPos(3),handles.previewImageMaxHeight*figPos(4)*ratio)/figPos(3);
    height = min(handles.previewImageMaxHeight*figPos(4),handles.previewImageMaxWidth*figPos(3)/ratio)/figPos(4);
    pos = get(handles.axes_image3,'Position');
    set(handles.axes_image3,'Position',[pos(1:2) width height]);
    axes(handles.axes_image3);
    colormap(gray(256));
    imagesc(handles.refImg,'Parent',handles.axes_image3);
    axis off;
end

axes(handles.axes_image1);
colormap(gray(256));
image(getSelectedImage(handles.panel_selStep1,handles),'Parent',handles.axes_image1);
axis off;

axes(handles.axes_image2);
colormap(gray(256));
image(getSelectedImage(handles.panel_selStep2,handles),'Parent',handles.axes_image2);
axis off;
end

% --- Executes on slider movement.
function slider1_Callback(hObject, eventdata, handles)
% hObject    handle to slider1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'Value') returns position of slider
%        get(hObject,'Min') and get(hObject,'Max') to determine range of slider


% --- Executes during object creation, after setting all properties.
function slider1_CreateFcn(hObject, eventdata, handles)
% hObject    handle to slider1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: slider controls usually have a light gray background.
if isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor',[.9 .9 .9]);
end


% --- Executes on slider movement.
function slider3_Callback(hObject, eventdata, handles)
% hObject    handle to slider3 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'Value') returns position of slider
%        get(hObject,'Min') and get(hObject,'Max') to determine range of slider


% --- Executes during object creation, after setting all properties.
function slider3_CreateFcn(hObject, eventdata, handles)
% hObject    handle to slider3 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: slider controls usually have a light gray background.
if isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor',[.9 .9 .9]);
end

% --------------------------------------------------------------------
function menu_setting_Callback(hObject, eventdata, handles)
% hObject    handle to menu_setting (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
