function [idList,createGuiMap,formatArgsMap,procFilterMap,editFilterMap] = initFilters()
%INITFILTERS Initializes the filter set
%   Registers the filters with their respective GUI, formatting and
%   procesing functions

% Copyright (c) 2010 
% Andre Dau, Athanasios Karamalis, Christian Wachinger 
% Computer Aided Medical Procedures (CAMP) 
% Technische Universitt Mnchen (TUM) 
% http://campar.in.tum.de/Students/RFtoB 
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are met:
%    * Redistributions of source code must retain the above copyright
%       notice, this list of conditions and the following disclaimer.
%    * Redistributions in binary form must reproduce the above copyright
%       notice, this list of conditions and the following disclaimer in the
%       documentation and/or other materials provided with the distribution.
%    * Neither the name of CAMP, TUM nor the
%       names of its contributors may be used to endorse or promote products
%       derived from this software without specific prior written permission.
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
% ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
% WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
% DISCLAIMED. IN NO EVENT SHALL Andre Dau, Athanasios Karamalis, Christian Wachinger BE LIABLE FOR ANY
% DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
% (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
% LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
% ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
% (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


    createGuiMap = containers.Map();
    formatArgsMap = containers.Map();
    procFilterMap = containers.Map();
    editFilterMap = containers.Map();
    idList = {};
    function registerFilter(id,createGuiFct,formatArgsFct,procFilterFct,editFilterFct)
        idList = [idList;{id}];
        createGuiMap(id) = createGuiFct;
        formatArgsMap(id) = formatArgsFct;
        procFilterMap(id) = procFilterFct;
        editFilterMap(id) = editFilterFct;
    end

%Add paths to the filter subdirs
currFold = pwd;
addpath([currFold '/filters/hilbert']);
addpath([currFold '/filters/abs']);
addpath([currFold '/filters/fir_parks_mcclellan']);
addpath([currFold '/filters/log']);
addpath([currFold '/filters/gaussian']);
addpath([currFold '/filters/anisotropic_diffusion']);
addpath([currFold '/filters/mucro']);
addpath([currFold '/filters/threshold']);
addpath([currFold '/filters/scanconversion']);
%Registers the filters
registerFilter('Hilbert',@hilbert_createGui,@hilbert_formatArgs,@hilbert_procFilter,@hilbert_editFilter);
registerFilter('Absolute',@abs_createGui,@abs_formatArgs,@abs_procFilter,@abs_editFilter);
registerFilter('FIR - Parks-McClellan',@fir_parks_mcclellan_createGui,@fir_parks_mcclellan_formatArgs,@fir_parks_mcclellan_procFilter,@fir_parks_mcclellan_editFilter);
registerFilter('Log-Compression',@log_createGui,@log_formatArgs,@log_procFilter,@log_editFilter);
registerFilter('Gaussian lowpass blur',@gaussian_createGui,@gaussian_formatArgs,@gaussian_procFilter,@gaussian_editFilter);
registerFilter('Anisotropic diffusion',@anisotropic_diffusion_createGui,@anisotropic_diffusion_formatArgs,@anisotropic_diffusion_procFilter,@anisotropic_diffusion_editFilter);
registerFilter('Mucro',@mucro_createGui,@mucro_formatArgs,@mucro_procFilter,@mucro_editFilter);
registerFilter('Threshold',@threshold_createGui,@threshold_formatArgs,@threshold_procFilter,@threshold_editFilter);

end

